from FLAI import data
import bnlearn as bn
import networkx as nx
import matplotlib.pyplot as plt
from pgmpy.factors.discrete import TabularCPD
import numpy as np
import itertools
import math
class CausalGraph():
    """
    The CausalGraph class represents a causal Bayesian network and provides methods for its manipulation, 
    inference, mitigation, and learning.
    """
    def __init__(self, flai_dataset = None, node_edge = None, CPD = None, indepence_test = True,
                 root_node = None, target = None,  verbose = 0):
        
        """
        Initialize a CausalGraph object.
        
        Args:
        flai_dataset (flai.data.Data, optional): Dataset to be used for the network. Default is None.
        node_edge (list of tuples, optional): List of tuples where each tuple represents an edge of the network. Default is None.
        CPD (list of TabularCPD objects, optional): List of conditional probability distributions for the network. Default is None.
        indepence_test (bool, optional): If true, performs an independence test on the network. Default is True.
        root_node (str, optional): The root node of the network. Default is None.
        target (str, optional): The target node of the network. Default is None.
        verbose (int, optional): Verbosity level. Default is 0.
        """

        if flai_dataset is None and node_edge is None:
            raise Exception("Data or edges should be provided")
        if target is None:
            raise Exception("Please indicate the target feature")
        if node_edge is None:
            if not isinstance(flai_dataset,data.Data):
                raise Exception("Data should be a FLAI.data.Data class")
            self.flai_dataset = flai_dataset
            if root_node is None:
                self.graph = bn.structure_learning.fit(flai_dataset.data, methodtype='hc', scoretype='bic',verbose= verbose)
            else:
                self.graph = bn.structure_learning.fit(flai_dataset.data,root_node= root_node,verbose= verbose)
            self.graph['model_edges'] = [(edge[0],edge[1]) if edge[0] != target else (edge[1],edge[0]) for edge in self.graph['model_edges']]
            list_edges = self.graph['model_edges']
            ends = list((np.array(self.graph['model_edges'])[:,0]))
            for n in list(self.graph['model'].nodes()):
                if (n not in ends) & (n != target):
                    print(n)
                    list_edges = list_edges + [(n,target)]
            self.graph['model_edges'] = list_edges
            if indepence_test:
                self.independence_test(flai_dataset, test='chi_square', prune=True,verbose= verbose)
        else:
            if CPD is None:
                self.flai_dataset = flai_dataset
                self.graph = bn.make_DAG(node_edge, verbose= verbose)
            else:
                self.flai_dataset = flai_dataset
                self.graph = bn.make_DAG(node_edge, CPD=CPD,verbose= verbose)


    def independence_test(self, flai_dataset, test='chi_square', prune=True,verbose= 0):
        """
        Performs an independence test on the network.
        
        Args:
        flai_dataset (flai.data.Data): Dataset to be used for the independence test.
        test (str, optional): Type of test to be performed. Default is 'chi_square'.
        prune (bool, optional): If true, prunes the network after the test. Default is True.
        verbose (int, optional): Verbosity level. Default is 0.
        """

        self.graph = bn.independence_test(self.graph, flai_dataset.data, test, prune,verbose= verbose)

    def inference(self,variables = [],evidence = {},verbose=0):
        """
        Performs inference on the network given evidence.

        Args:
        variables (list, optional): List of variables to be included in the inference. Default is an empty list.
        evidence (dict, optional): Dictionary of evidence in the form {variable: value}. Default is an empty dictionary.
        """

        if len(variables) == 0:
            raise Exception("Variables should be provided") 
        return bn.inference.fit(self.graph, variables=variables, evidence=evidence,verbose = verbose).df          

    def predict(self, data = None, variables = [],verbose=0):
        """
        Makes predictions based on the network.

        Args:
        data (DataFrame, optional): Data used for making predictions. Default is None.
        variables (list, optional): List of variables to be predicted. Default is an empty list.
        """
        if len(variables) == 0:
            raise Exception("Variables should be provided") 
        if data is None:
            raise Exception("Data should be provided") 
        return bn.predict(self.graph, data, variables = variables, verbose=verbose)
    
    def learn_cpd(self, flai_dataset = None, methodtype= None,verbose = 0):
        """
        Learn the Conditional Probability Distribution (CPD) for the network.

        Args:
        flai_dataset (flai.data.Data, optional): Dataset to be used for learning the CPD. Default is None.
        methodtype (str, optional): The method type to be used for learning. Default is None.
        verbose (int, optional): Verbosity level. Default is 0.
        """

        if methodtype is None:
            methodtype = 'bayes'
        if flai_dataset is None and self.flai_dataset is None:
            raise Exception("Data should be provided")
        if flai_dataset is None: 
            self.graph = bn.parameter_learning.fit(self.graph,  self.flai_dataset.data,methodtype = methodtype,verbose= verbose) 
        else:
            if not isinstance(flai_dataset,data.Data):
                raise Exception("Data should be a FLAI.data.Data class")
            self.flai_dataset = flai_dataset
            self.graph = bn.parameter_learning.fit(self.graph, self.flai_dataset.data,methodtype = methodtype,verbose= verbose) 

    def calculate_cpd(self,verbose = 0):
        """
        Calculate the Conditional Probability Distribution (CPD) for the network.

        Args:
        verbose (int, optional): Verbosity level. Default is 0.
        """

        list_cpd = []
        for node in list(self.graph['model'].nodes()):
            node_value = list(np.sort(self.flai_dataset.data[node].unique()))
            evidence = [edge[0] for edge in self.graph['model_edges'] if edge[1] == node]
            evidence_value = [list(np.sort(self.flai_dataset.data[e].unique())) for e in evidence]
                
            evidence_combination = evidence_value
            evidence_combination = list(itertools.product(*evidence_combination))
            list_probas = []
            for ec in evidence_combination:
                filters = [[filter[0],filter[1]]  for filter in zip(evidence,ec)]
                data_aux =  self.flai_dataset.data
                for f in filters:
                    data_aux = data_aux[data_aux[f[0]] == f[1]]
                list_probas = list_probas + [[0.5 if math.isnan((data_aux[node] == nv).sum() / len(data_aux[node] == nv)) else (data_aux[node] == nv).sum() / len(data_aux[node] == nv) for nv in node_value]]
            list_cpd = list_cpd + [TabularCPD(node, len(node_value), np.array(list_probas).T.tolist(),
                        evidence= evidence,
                        evidence_card= [len(ev) for ev in evidence_value])]
        self.graph = bn.make_DAG(list(self.graph['model_edges']), CPD=list_cpd,verbose= verbose)
    def plot(self, directed = True):
        """
        Plots the network.

        Args:
        directed (bool, optional): If True, plots a directed graph. Default is True.
        """

        if directed:
            G = nx.DiGraph()
        else:
            G = nx.Graph()
        G.add_edges_from(self.graph['model_edges'])
        plt.figure(figsize=(5,5)) 
        nx.draw_circular(
            G, with_labels=True, arrowsize=30, node_size=800, 
            alpha=0.3, font_weight="bold"
        )

        #Graph(G, node_layout='radial', node_shape= 'o',
        #node_labels=True ,node_size=10, arrows = directed
        #    )
        
    
    def get_CPDs(self):
        """
        Returns the Conditional Probability Distributions (CPDs) of the network.
        """

        CPDs = bn.print_CPD(self.graph,verbose = 0)
        return CPDs

    def generate_dataset(self, n_samples = 1000, methodtype = 'bayes', verbose = 0):
        """
        Generates a dataset based on the network.

        Args:
        n_samples (int, optional): Number of samples to generate. Default is 1000.
        methodtype (str, optional): The method type to be used for generation. Default is 'bayes'.
        verbose (int, optional): Verbosity level. Default is 0.
        """

        df = bn.sampling(self.graph, n= n_samples, methodtype = methodtype, verbose = verbose)
        return data.Data(df, transform=False)

    def mitigate_edge_relation(self, sensible_feature = []):
        """
        Mitigates the relationship of certain edges in the network.

        Args:
        sensible_feature (list, optional): List of features for which to mitigate the relationship. Default is an empty list.
        """

        ### improve for more than one sensible feature 
        if len(sensible_feature) == 0:
            raise Exception("Sensible features should be provided")
        reverse_edge = []
        ### first apply transitivity and reverse
        maintain_edge = self.graph['model_edges']
        longitud = -1
        while(longitud!=0):
            model_edges = maintain_edge
            maintain_edge = [e for e in model_edges if ((e[1] not in  sensible_feature) & (e[0] in sensible_feature)) | ((e[1] not in sensible_feature) & (e[0] not in sensible_feature))] 
            #print('maintain',maintain_edge)
            longitud = 0

            for sf in sensible_feature:
                impact_sensible = [e for e in model_edges if e[1] == sf ]
                longitud = longitud + len(impact_sensible)
                #print(impact_sensible)
                for i_s in impact_sensible:
                    sensible_impact = [e for e in model_edges if e[0] == sf ]
                    #print(sensible_impact)
                    transitivity_edge = [(i_s[0],e[1]) for e in model_edges if (i_s[0] != e[1]) & (e[0] == sf) & ((i_s[0],e[1]) not in reverse_edge) ]
                    #print('Transitivity',transitivity_edge)
                    maintain_edge = maintain_edge + transitivity_edge
                    if ((i_s[0], 'label') in maintain_edge) & ((sf, i_s[0]) not in reverse_edge):
                        maintain_edge = maintain_edge + [(sf, i_s[0])]
                        reverse_edge = reverse_edge + [(i_s[0],sf)]
            maintain_edge_aux = maintain_edge
            maintain_edge = []
            for mea in maintain_edge_aux:
                if mea not in maintain_edge:
                    maintain_edge = maintain_edge + [mea]
            #print(longitud)
            #print('maintain',maintain_edge)
        self.graph['model_edges'] = maintain_edge
        return maintain_edge
    def mitigate_calculation_cpd(self,verbose = 0,sensible_feature = []):
        """
        Mitigates the calculation of the Conditional Probability Distribution (CPD) for certain features.

        Args:
        verbose (int, optional): Verbosity level. Default is 0.
        sensible_feature (list, optional): List of sensible features for which to mitigate the CPD calculation. Default is an empty list.
        """
        if len(sensible_feature) == 0:
            raise Exception("Sensible feature should be provided")
        list_cpd = []
        for node in list(self.graph['model'].nodes()):
            node_value = list(np.sort(self.flai_dataset.data[node].unique()))
            evidence = [edge[0] for edge in self.graph['model_edges'] if edge[1] == node]
            evidence_value = [list(np.sort(self.flai_dataset.data[e].unique())) for e in evidence]
                
            evidence_combination = evidence_value
            evidence_combination = list(itertools.product(*evidence_combination))
            list_probas = []
            for ec in evidence_combination:
                filters = [[filter[0],filter[1]]  for filter in zip(evidence,ec)]
                data_aux =  self.flai_dataset.data
                for f in filters:
                    if f[0] not in sensible_feature :
                        data_aux = data_aux[data_aux[f[0]] == f[1]]
                if node in sensible_feature:
                    list_probas = list_probas + [[1/len(node_value) for nv in node_value]]
                else:
                    list_probas = list_probas + [[0.5 if math.isnan((data_aux[node] == nv).sum() / len(data_aux[node] == nv)) else (data_aux[node] == nv).sum() / len(data_aux[node] == nv) for nv in node_value]]
            list_cpd = list_cpd + [TabularCPD(node, len(node_value), np.array(list_probas).T.tolist(),
                        evidence= evidence,
                        evidence_card= [len(ev) for ev in evidence_value])]
        self.graph =  bn.make_DAG(list(self.graph['model_edges']), CPD=list_cpd,verbose= verbose)
        


import pandas as pd
import bnlearn as bn
from sklearn.metrics import accuracy_score,confusion_matrix
from sklearn.preprocessing import OrdinalEncoder
import numpy as np
import matplotlib.pyplot as plt
import math
plt.style.use("seaborn-dark-palette")

class Data():
    def __init__(self, data = None, transform = True, verbose = 0):
        """
        Initialize the Data class.

        Args:
        data (DataFrame, optional): The data to be used. If None, an exception is raised. Default is None.
        transform (bool, optional): If True, the data is transformed to numerical form. Default is True.
        verbose (int, optional): Verbosity level. Default is 0.
        """
        if data is None:
            raise Exception("Data is not provided")
        self.data = data
        self.transform = transform
        if self.transform:
            self.transform_data_numeric()

    def transform_data_numeric(self, verbose = 0):
        """
        Transform the data to numerical form.

        Args:
        verbose (int, optional): Verbosity level. Default is 0.
        """

        if self.data is None:
            raise Exception("Data is not provided")
        dfhot, dfnum = bn.df2onehot(self.data,verbose = verbose)
        #self.data2 = dfnum
        ### Add the transform map to comeback.
        enc = OrdinalEncoder()
        enc.fit(self.data)
        self.map_cat = enc.categories_
        self.data = pd.DataFrame(enc.transform(self.data), columns = self.data.columns)
    def fairness_metrics(self, target_column = None, predicted_column = None, 
                        columns_fair = None):
        """
        Calculate fairness for a subgroup of population.

        Args:
        target_column (str, optional): The target column. If None, an exception is raised. Default is None.
        predicted_column (str, optional): The predicted column. If None, an exception is raised. Default is None.
        columns_fair (list, optional): List of column names to consider for fairness. Default is None.
        """

        if target_column is None:
            raise Exception("target_column is not provided")
        if predicted_column is None:
            raise Exception("predicted_column is not provided")
        #Confusion Matrix
        result = {}
        result.update({'model' : self.metrics(target_column, predicted_column)})
        
        if not columns_fair is None:
            for c in columns_fair.keys():
                privileged = self.metrics(target_column, predicted_column, {'name' : c, 'value': columns_fair[c]['privileged']})
                unprivileged = self.metrics(target_column, predicted_column, {'name' : c, 'value': columns_fair[c]['unprivileged']})
                #disparate_impact = len(self.data[(self.data[predicted_column] == 1) & (self.data[c] == columns_fair[c]['unprivileged'])])  / len(self.data[(self.data[predicted_column] == 1) & (self.data[c] == columns_fair[c]['privileged'])]) 
                result.update({c: {'privileged' : privileged, 'unprivileged' : unprivileged,
                                   'fair_metrics' : {'EOD' : unprivileged['TPR'] - privileged['TPR'],
                                                     'DI' : unprivileged['PPP'] / privileged['PPP'],
                                                     'SPD' :  unprivileged['PPP'] - privileged['PPP'],
                                                     'OD' :  (unprivileged['FPR'] - privileged['FPR']) + (unprivileged['TPR'] - privileged['TPR']),
                                                     }}})

        return result
    def theil_index(self, y_true = None, y_pred = None, value_pred = None):
        """
        Calculate the Theil index for the prediction.

        Args:
        y_true (array, optional): The true labels. If None, an exception is raised. Default is None.
        y_pred (array, optional): The predicted labels. If None, an exception is raised. Default is None.
        value_pred (int, optional): The value to predict. If None, an exception is raised. Default is None.
        """

        if y_true is None:
            raise Exception("y_true is not provided")
        if y_pred is None:
            raise Exception("y_pred is not provided")
        if value_pred is None:
            raise Exception("value_pred is not provided")
        
        y_pred = y_pred.ravel()
        y_true = y_true.ravel()
        y_pred = (y_pred == value_pred).astype(np.float64)
        y_true = (y_true == value_pred).astype(np.float64)
        b = 1 + y_pred - y_true
        return np.mean(np.log((b / np.mean(b))**b) / np.mean(b))
           
    def metrics(self, target_column = None, predicted_column = None, column_filter = None):
        """
        Calculate various metrics for the prediction.

        Args:
        target_column (str, optional): The target column. If None, an exception is raised. Default is None.
        predicted_column (str, optional): The predicted column. If None, an exception is raised. Default is None.
        column_filter (dict, optional): Dictionary with keys as column names and values as filters. Default is None.
        """

        if target_column is None:
            raise Exception("target_column is not provided")
        if predicted_column is None:
            raise Exception("predicted_column is not provided")
        if column_filter is None:
            data = self.data
        else:
            print('Calculating metrics for :', column_filter['name'],' the value : ', column_filter['value'])
            data = self.data[self.data[column_filter['name']] == column_filter['value']]

        cm=confusion_matrix(data[target_column],data[predicted_column])
        ti = self.theil_index(data[target_column],data[predicted_column],1)
        TN, FP, FN, TP = cm.ravel()
        
        N = TP+FP+FN+TN #Total population
        ACC = (TP+TN)/N #Accuracy
        TPR = TP/(TP+FN) # True positive rate
        FPR = FP/(FP+TN) # False positive rate
        FNR = FN/(TP+FN) # False negative rate
        PPP = (TP + FP)/N # % predicted as positive
        return {'ACC' : ACC, 'TN' : TN, 'FP' : FP, 'FN' : FN, 'TP' : TP,'TPR' : TPR, 'FPR': FPR, 'FNR' : FNR, 'PPP' : PPP }
    def get_df_metrics(self, metrics_json = None):
        """
        Get the performance and fairness metrics as dataframes.

        Args:
        metrics_json (json, optional): The metrics in json format. If None, an exception is raised. Default is None.
        """
        if metrics_json is None:
            raise Exception("metrics_json is not provided")
        df_performance = pd.DataFrame(columns = ['ACC', 'TN', 'FP', 'FN', 'TP', 'TPR', 'FPR', 'FNR', 'PPP'])
        df_fairness = pd.DataFrame(columns = ['EOD', 'DI', 'SPD', 'OD'])
        for k in metrics_json.keys():
            if k == 'model':
                df_performance.loc['model'] = metrics_json['model'].values()
            else:
                df_performance.loc[k+'_privileged'] = metrics_json[k]['privileged'].values()
                df_performance.loc[k+'_unprivileged'] = metrics_json[k]['unprivileged'].values()
                df_fairness.loc[k+'_fair_metrics'] = metrics_json[k]['fair_metrics'].values()
        return df_performance,df_fairness
    
    def fairness_eqa_eqi(self, features = None, target_column = None, column_filter = None,plot = True):
        """
        Calculate fairness metrics for the data.

        Args:
        target_column (str, optional): The target column. If None, an exception is raised. Default is None.
        features (dict, optional): Dictionary with keys as column names as feature. Default is None.
        column_filter (dict, optional): Dictionary with keys as column names as sensible. Default is None.
        """

        if target_column is None:
            raise Exception("target_column is not provided")
        if features is None:
            raise Exception("features is not provided")
        if column_filter is None:
            raise Exception("predicted_column is not column_filter")
  
  
        df_aux = self.data.groupby(by=column_filter + features).agg({target_column: ['count', 'sum']})
        df_aux_ideal = self.data.groupby(by=features).agg({target_column: ['count', 'sum']})
        df_aux.columns = df_aux.columns.droplevel(0)
        df_aux = df_aux.reset_index()
        combinations_s = df_aux[column_filter].value_counts().index.values
        df_aux = df_aux.set_index(column_filter + features)

        df_aux_ideal.columns = df_aux_ideal.columns.droplevel(0)
        df_aux_ideal = df_aux_ideal.reset_index()
        combinations_f = df_aux_ideal[features].value_counts().index.values
        df_aux_ideal = df_aux_ideal.set_index(features)

        df_aux_ideal['px'] = df_aux_ideal['sum'] / df_aux_ideal['count']
        df_aux_ideal = df_aux_ideal.sort_values(by='px')
        df_aux_ideal['dx'] = [0] + (df_aux_ideal['count'].cumsum() / df_aux_ideal['count'].sum()).tolist()[:-1]
        df_aux['px'] = df_aux['sum'] / df_aux['count']


        n_group = combinations_s.shape[0]
        groups = [str(column_filter) + str(s) for s in combinations_s]
        combinations = [[s + f for s in combinations_s] for f in combinations_f]
        for c,f in zip(combinations,combinations_f):
            for n in range(n_group):
                df_aux_ideal.loc[f,'px_'+str(n)] = df_aux.loc[c[n]]['px']
                df_aux_ideal.loc[f,'count_'+str(n)] = df_aux.loc[c[n]]['count']
                df_aux_ideal['dx_'+str(n)] = [0] + (df_aux_ideal['count_'+str(n)].cumsum() / df_aux_ideal['count_'+str(n)].sum()).tolist()[:-1]
                
        if plot:
            self.plot_fairness_eqa_eqi(df_aux_ideal,n_group,groups)
        n_p = -1
        p_max = 0
        d_max = 0
        for n in range(n_group):
            p_aux = df_aux_ideal['px_'+str(n)].max()
            d_aux = df_aux_ideal['dx_'+str(n)].max()
            if p_aux > p_max:
                p_max = p_aux
                d_max = d_aux
                n_p = n
            elif p_aux == p_max:
                if d_aux < d_max:
                    p_max = p_aux
                    d_max = d_aux
                    n_p = n

        df_f = pd.DataFrame(columns = ['group','reference','EQI','EQA','F'])
        for n in range(n_group):
            if n != n_p:
                eqi = (df_aux_ideal['dx_'+str(n_p)] - df_aux_ideal['dx_'+str(n)]).values
                eqa = (df_aux_ideal['px_'+str(n_p)] - df_aux_ideal['px_'+str(n)]).values

                EQI = np.round(eqi.mean(),2)
                EQA = np.round(eqa.mean(),2)
                F = np.round(math.sqrt(EQA**2 + EQI**2),2)
                df_f.loc[n] = [groups[n],groups[n_p],EQI,EQA,F]
        return df_f,df_aux_ideal
                
    def plot_fairness_eqa_eqi(self, df_aux_ideal,n_group,groups):
        fig = plt.figure(figsize=(20, 12))
        fig.subplots_adjust(hspace=0.8)
        gs = fig.add_gridspec(6,4)

        ax1 = fig.add_subplot(gs[0:2, 0:2])
        ax2 = fig.add_subplot(gs[0:2, 2:4])
        ax3 = fig.add_subplot(gs[2:6, :])

        ### plot dx
        ax1.set_title('Distribution',fontsize="20")
        ax1.set_xlabel('Feature Vector',fontsize="20")
        ax1.set_ylabel('EQI',fontsize="20")
        for n in range(n_group):
            ax1.plot(range(df_aux_ideal.shape[0]), 
                 df_aux_ideal['dx_'+str(n)], '-',linestyle='dashed',  marker='o',markersize=10,linewidth=2,label=groups[n])
        ax1.legend(fontsize="20")

        ### plot px
        ax2.set_title('Probability',fontsize="20")
        ax2.set_xlabel('Feature Vector',fontsize="20")
        ax2.set_ylabel('EQA',fontsize="20")
        for n in range(n_group):
            ax2.plot(range(df_aux_ideal.shape[0]), 
                 df_aux_ideal['px_'+str(n)], '-',linestyle='dashed',  marker='o',markersize=10,linewidth=2,label=groups[n])
        ax2.legend(fontsize="20")

        ### plot curve
        ax3.set_title('Fairness Curve',fontsize="20")
        ax3.set_xlabel('EQI',fontsize="20")
        ax3.set_ylabel('EQA',fontsize="20")
        for n in range(n_group):
            ax3.plot(df_aux_ideal['dx_'+str(n)], 
                 df_aux_ideal['px_'+str(n)], '-',linestyle='dashed',  marker='o',markersize=10,linewidth=2,label=groups[n])
        ax3.legend(fontsize="20")


        plt.show()


import pathlib
from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent

VERSION = '2.0.0' #Muy importante, deberéis ir cambiando la versión de vuestra librería según incluyáis nuevas funcionalidades
PACKAGE_NAME = 'FLAI_CAUSAL' #Debe coincidir con el nombre de la carpeta 
AUTHOR = 'Rubén González Sendino' #Modificar con vuestros datos
AUTHOR_EMAIL = 'rubo.g@icloud.com' #Modificar con vuestros datos
URL = 'https://github.com/rugonzs/FLAI' #Modificar con vuestros datos

LICENSE = 'Apache-2.0 license' #Tipo de licencia
DESCRIPTION = 'Library to creat causal model and mitigate the bias.' #Descripción corta
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8') #Referencia al documento README con una descripción más elaborada
LONG_DESC_TYPE = "text/markdown"
PROJECT_URLS = {
    "Documentation": "https://rugonzs.github.io/FLAI/",
    "Source Code": 'https://github.com/rugonzs/FLAI',
}

#Paquetes necesarios para que funcione la libreía. Se instalarán a la vez si no lo tuvieras ya instalado
INSTALL_REQUIRES = [
      'bnlearn==0.7.8','networkx==2.8.8','matplotlib==3.6.2','pgmpy==0.1.20','numpy==1.23.4', 'pandas==1.5.1','scikit-learn==1.0.2','scipy==1.9.3','statsmodels==0.13.5'
      ]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    project_urls=PROJECT_URLS,
    install_requires=INSTALL_REQUIRES,
    license=LICENSE,
    packages=find_packages(),
    include_package_data=True
)
